/*
udp-log-win32.c: send debug messages over udp
ed <irc.rizon.net>, 2020-04-10, v3, MIT-Licensed

https://ocv.me/dev/?udp-log-win32.c (windows)
https://ocv.me/dev/?udp-log-unix.c  (linux/osx)

udp_msg() takes tchar input (ascii or unicode depending on compiler),
the udp packets are utf-8
*/


// Properties » Configuration Properties » C/C++ »
//   Precompiled Header » Not Using Precompiled Headers
#include <SDKDDKVer.h>
#include <tchar.h>
#include <string>

//#define _WIN32_WINNT 0x0600
#include <WinSock2.h>
#include <WS2tcpip.h>
#pragma comment(lib, "ws2_32.lib")

#include <cstdarg>
#include <cstring>
#include <cstdio>
#include <cstdlib>


// sends "<sourcefile>  lineno  [function]"
#define IM_HERE udp_msg(_T("<%s> %4d @ [%s]\n"), _T(__FILE__), __LINE__, _T(__FUNCTION__))

// sends "<sourcefile>  lineno  your-arguments"
#define DBG(fmt, ...) udp_msg(_T("<%s> %4d ") _T(fmt) _T("\n"), _T(__FILE__), __LINE__, __VA_ARGS__)



char* tchar2utf8(const _TCHAR* wstr) {
	// takes a _TCHAR* and converts it to utf8,
	// only does ansi/unicode, no mbcs support
#ifdef _UNICODE
	int bufsz = ::WideCharToMultiByte(CP_UTF8, 0, wstr, -1, NULL, 0, NULL, NULL);
	char* ret = (char*)malloc(bufsz);
	::WideCharToMultiByte(CP_UTF8, 0, wstr, -1, ret, bufsz, NULL, NULL);
	return ret;
#else
	int bufsz = strlen(wstr) + 1;
	char* ret = (char*)malloc(bufsz);
	memcpy_s(ret, bufsz, wstr, bufsz);
	return ret;
#endif
}


void udp_msg(const _TCHAR* fmt, ...) {
	const int port = 42829;
	const char* src_ip = "127.0.0.1";
	const char* dst_ip = "127.0.0.1";
	int len = 1024; //will realloc if necessary

	va_list args;
	_TCHAR* buf = (_TCHAR*)malloc(len * sizeof(TCHAR));

	va_start(args, fmt);
	len = _vsntprintf_s(buf, len, _TRUNCATE, fmt, args);
	va_end(args);

	if (len == 0) {
		va_start(args, fmt);
		len = _vsctprintf(fmt, args);
		va_end(args);

		free(buf);
		buf = (_TCHAR*)malloc(len * sizeof(TCHAR));

		va_start(args, fmt);
		_vsntprintf_s(buf, len, _TRUNCATE, fmt, args);
		va_end(args);
	}

	char* utf = tchar2utf8(buf);
	len = (int)strlen(utf);

	WSAData data;
	WSAStartup(MAKEWORD(2, 2), &data);

	sockaddr_in src, dst;
	memset(&src, 0, sizeof(src));
	memset(&dst, 0, sizeof(dst));

	src.sin_family = AF_INET;
	inet_pton(AF_INET, src_ip, &src.sin_addr.s_addr);
	src.sin_port = htons(0);

	dst.sin_family = AF_INET;
	inet_pton(AF_INET, dst_ip, &dst.sin_addr.s_addr);
	dst.sin_port = htons(port);

	SOCKET s = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	bind(s, (sockaddr*)&src, sizeof(src));
	sendto(s, utf, len, 0, (sockaddr*)&dst, sizeof(dst));

	closesocket(s);
	WSACleanup();
	free(buf);
}


int perftest() {
	//WSAData data;
	//WSAStartup(MAKEWORD(2, 2), &data);

	for (int a = 0; a < 100 * 1000; a++)
		udp_msg(_T("hello world fgsfds dsfargeg %d\n"), a);

	return 0;
}


int _tmain(int argc, _TCHAR* argv[]) {
	//return perftest();

	IM_HERE;
	for (int a = 0; a < argc; a++)
		DBG("[%s]\n", argv[a]);

	return 0;
}


// receive messages in msys2 using:
//   socat -u udp4-recv:42829 -


// perftest results:
//  100k in 51.7 sec with WSAStartup/WSACleanup inside udp_msg() ==  1934/sec
// 1000k in 39.3 sec with WSAStartup just once inside perftest() == 25445/sec
//
// tested in PS using: Measure-Command {.\sepples_w32cli.exe}

